//
//  AuthenticationManager.swift
//  Serloc
//
//  Created by Joaquim Calvo Gubianas on 01/05/2018.
//  Copyright © 2018 Joaquim Calvo Gubianas. All rights reserved.
//

import Foundation
import p2_OAuth2

class AuthenticationManager {
    
    private static var instance : AuthenticationManager? = nil
    
    public let oAuthPasswordGrant : OAuth2PasswordGrant
    public let oAuthDataLoader : OAuth2DataLoader
    
    private var userAuthenticated = false
    private var isAuthenticating = false
    
    /**
    * Encapsulate the authentication manager constructor to implement
    * singleton pattern.
    */
    private init(UserEmail email : String, UserPassword password : String, RedirectView redirect : UIViewController,
                 FromView from : UIViewController) {
        self.oAuthPasswordGrant = OAuth2PasswordGrant(settings: [
            "client_id" : "spring-security-oauth2-read-write-client",
            "client_secret" : "spring-security-oauth2-read-write-client-password1234",
            "token_uri" : "http://localhost:8080/oauth/token",
            "username" : email,
            "password" : password,
            "keychain" : false] as OAuth2JSON)
        
        self.oAuthPasswordGrant.logger = OAuth2DebugLogger(.debug)
        self.oAuthDataLoader = OAuth2DataLoader(oauth2: oAuthPasswordGrant)
        
        //Try to authenticate
        self.isAuthenticating = true
        self.oAuthPasswordGrant.authorize(params: nil) { (json, error) in
            self.isAuthenticating = false
            if (error == nil) {
                self.userAuthenticated = true
                from.present(redirect, animated: false, completion: nil)
            } else {
                let alert = UIAlertController(title: "Login failure", message: "Invalid username or password", preferredStyle: .alert)
                alert.addAction(UIAlertAction(title: "Okei", style: .default, handler: nil))
                
                from.present(alert, animated: true)
            }
        }
        
        
    }
    
    /**
    * Return the initializated instance
    */
    public static func getInstance() -> AuthenticationManager {
        return AuthenticationManager.instance!
    }
    
    /**
    * Initialize a new authentication manager instance authenticating
    * the user with email and password.
    */
    public static func authenticate(UserEmail email : String, UserPassword password : String, RedirectView redirect : UIViewController,
                                    FromView from : UIViewController) {
        AuthenticationManager.instance = AuthenticationManager(UserEmail: email, UserPassword: password, RedirectView: redirect, FromView: from)
    }
    
    /**
    * Method to ensure is the authentication process is already in progress.
    */
    public static func isUserAuthenticating() -> Bool {
        return (self.instance?.isAuthenticating)!
    }
    
    /**
    * Method to evaluate if the authentication process finishes and if
    * the user is authenticated succesfully.
    */
    public static func isUserAuthenticated() -> Bool {
        return self.instance?.isAuthenticating == false && (self.instance?.userAuthenticated)!
    }
}
