import UIKit

class MainProfileViewController: UIViewController {
    
    private var services : [Service]?

    @IBOutlet weak var userName: UITextView!
    @IBOutlet weak var userDescription: UITextView!
    @IBOutlet weak var userPhoneNumber: UITextView!
    @IBOutlet weak var userEmail: UITextView!
    
    @IBOutlet weak var userInformationLoader: UIActivityIndicatorView!
    
    
    override func viewDidLoad() {
        super.viewDidLoad()

        self.title = "Profile"
        
        self.loadUserInformation()
    }
    
    /**
    * Function which starts obtaining the user information.
    */
    private func loadUserInformation() {
        self.userName.isHidden = true
        self.userDescription.isHidden = true
        self.userPhoneNumber.isHidden = true
        self.userEmail.isHidden = true
        self.userInformationLoader.isHidden = false
        self.userInformationLoader.startAnimating()
        
        UserServiceConsumer.loadUserProfile(Controller : self)
    }
    
    /**
    * Function called from service consumer to redirect to the
    * manage services view.
    */
    public func redirectToManageServicesView(_ parsedServices : [Service]) {
        self.services = parsedServices
        self.performSegue(withIdentifier: "goToManageServicesView", sender: self)
    }

    /**
    * Pressed button to redirect to the user associated services.
    */
    @IBAction func goToManageServiceView(_ sender: Any) {
        SerlocServiceConsumer.getUserServices(ProfileViewController: self)
    }
    
    /**
    * Pressed button to redirect to the user edit profile view.
    */
    @IBAction func goToEditProfileView(_ sender: Any) {
        //TODO
    }
    
    /**
    * Function which performs a logical logout.
    */
    @IBAction func performLogout(_ sender: Any) {
        let loginController : UIViewController = (self.storyboard?.instantiateViewController(withIdentifier: "LoginController"))!
        
        self.present(loginController, animated: true, completion: nil)
    }
    
    /**
     * Modifying the segue preparatio to add arguments.
     */
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if (segue.identifier == "goToManageServicesView") {
            (segue.destination as! UserServicesListController).services = self.services
        }
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    /**
    * Function which loads the user information obtained from the server.
    */
    public func showUserInformation(UserInformation user : User) {
        
        self.userInformationLoader.startAnimating()
        
        self.userName.isHidden = false
        self.userDescription.isHidden = false
        self.userPhoneNumber.isHidden = false
        self.userEmail.isHidden = false
        self.userInformationLoader.isHidden = true
        
        self.userName.text = user.name
        self.userDescription.text = user.description
        self.userPhoneNumber.text = user.phone
        self.userEmail.text = user.email
    }

}
