//
//  NonAuthenticatedRequestManager.swift
//  Serloc
//
//  Created by Joaquim Calvo Gubianas on 05/05/2018.
//  Copyright © 2018 Joaquim Calvo Gubianas. All rights reserved.
//

import Foundation
import Alamofire

class NonAuthenticatedRequestsManager {
    
    /**
    * Send a create new user request to server
    * without any authentication.
    */
    public static func sendCreateNewUserRequest(NewUser newUser : User, FromView fromView : UIViewController, ToView toView : UIViewController) {
        
        let jsonEncoder : JSONEncoder = JSONEncoder()
        let newUserRequestBody : Data = try! jsonEncoder.encode(newUser)
        
        let createNewUserRequestUrl = URL(string : "http://localhost:8080/user")
        var createNewUserRequest = URLRequest(url: createNewUserRequestUrl!)
        createNewUserRequest.httpMethod = HTTPMethod.post.rawValue
        createNewUserRequest.setValue("application/json; charset=UTF-8", forHTTPHeaderField: "Content-Type")
        createNewUserRequest.httpBody = newUserRequestBody
        
        Alamofire.request(createNewUserRequest).responseString(completionHandler: { (responseString) in
            let error = responseString.result.error as? AFError
            if (error != nil) {
                //Show some message indicating error
                let alert = UIAlertController(title: "Error creating new user.", message: "Error on the create new user operation request. Try it later.", preferredStyle: .alert)
                alert.addAction(UIAlertAction(title: "Okei", style: .default, handler: nil))
                
                fromView.present(alert, animated: true)
            } else {
                //try to perform a login operation and then go to the next view.
                AuthenticationManager.authenticate(UserEmail: newUser.email, UserPassword: newUser.password, RedirectView: toView, FromView: fromView)
            }
        })
    }
}
