//
//  SerlocServiceConsumer.swift
//  Serloc
//
//  Created by Joaquim Calvo Gubianas on 09/05/2018.
//  Copyright © 2018 Joaquim Calvo Gubianas. All rights reserved.
//

import Foundation
import CoreLocation
import MapKit

/**
 * Class which is goint to integrate with the service
 * endpoints of the back end.
 */
class SerlocServiceConsumer {

    
    /**
    * Method to obtain the services getting a location.
    */
    public static func getServicesBy(Request getServicesRequest : ServicesRequest, ServiceTypeController serviceTypeController : ServiceTypeListController) {
        var parsedServices : Array<Service> = Array()
        
        let authManager = AuthenticationManager.getInstance()
        
        let urlComponents = NSURLComponents(string: "http://localhost:8080/secured/service")
        let serviceType : String = String(getServicesRequest.serviceType)
        
        let minLatitude = getServicesRequest.latitude - 0.1001
        let maxLatitude = getServicesRequest.latitude + 0.1001
        let minLongitude = getServicesRequest.longitude - 0.1001
        let maxLongitude = getServicesRequest.longitude + 0.1001
        
        let minLatitudeQueryItem = URLQueryItem(name: "minLatitude", value: minLatitude.description)
        let maxLatitudeQueryItem = URLQueryItem(name: "maxLatitude", value: maxLatitude.description)
        let minLongitudeQueryItem = URLQueryItem(name: "minLongitude", value: minLongitude.description)
        let maxLongitudeQueryItem = URLQueryItem(name: "maxLongitude", value: maxLongitude.description)
        let serviceTypeQueryItem = URLQueryItem(name: "serviceType", value: serviceType)
        let queryItems :[URLQueryItem] = [minLatitudeQueryItem, maxLatitudeQueryItem, minLongitudeQueryItem, maxLongitudeQueryItem, serviceTypeQueryItem]
        
        urlComponents?.queryItems = queryItems
        let url : URL = (urlComponents?.url)!
        
        
        let request = authManager.oAuthPasswordGrant.request(forURL: url)
        
        authManager.oAuthDataLoader.perform(request: request) { (response) in
            do {
                if (response.error == nil) {
                    let availableServices = try JSONSerialization.jsonObject(with: response.data!, options: JSONSerialization.ReadingOptions.allowFragments) as! [String : AnyObject]
                    
                    if let servicesAsJson = availableServices["services"] {
                        if servicesAsJson.count > 0 {
                            for index in 0...servicesAsJson.count-1 {
                                let serviceAsJson = (servicesAsJson as! NSArray)[index] as! [String : AnyObject]
                                
                                let service : Service = ServiceJSONParser.parseJsonToService(SerlocServiceFields: serviceAsJson)
                                parsedServices.append(service)
                            }
                        }
                    }
                    
                    DispatchQueue.main.async {
                        serviceTypeController.goToServicesList(ServicesList: parsedServices)
                    }
                }
            } catch let error {
                print("Error trying to obtain geolocated user services \(error.asOAuth2Error.description)")
            }
        }
    }
    
    
    /**
    * Get services defined by a concrete user.
    */
    public static func getUserServices(ProfileViewController profileViewController : MainProfileViewController) {
        
        var parsedServices = [Service]()
        
        let authManager = AuthenticationManager.getInstance()
        let getUserServicesUrl = URL(string: "http://localhost:8080/secured/user/services")
        
        let request = authManager.oAuthPasswordGrant.request(forURL: getUserServicesUrl!)
        
        authManager.oAuthDataLoader.perform(request: request) { (response) in
            do {
                //The request don't produce error
                if response.error == nil {
                    let availableServices = try JSONSerialization.jsonObject(with: response.data!, options: JSONSerialization.ReadingOptions.allowFragments) as! [String : AnyObject]
                    
                    if let servicesContent = availableServices["services"] {
                        if servicesContent.count > 0 {
                            for index in 0...servicesContent.count-1 {
                                let serviceAsJson = (servicesContent as! NSArray)[index] as! [String : AnyObject]
                                
                                let service : Service = ServiceJSONParser.parseJsonToService(SerlocServiceFields: serviceAsJson)
                                parsedServices.append(service)
                            }
                        }
                    }
                } else  {
                    print("Http error gettin the user services.")
                }
                
                DispatchQueue.main.async {
                    profileViewController.redirectToManageServicesView(parsedServices)
                }
                
            } catch let error {
                print("Error managing the server response \(error.asOAuth2Error.description)")
            }
        }
    }
    
    /**
    * Function which a map view displayed services.
    */
    public static func updateServicesMapVisibleServices(Request updateServicesRequest : UpdateServicesRequest, Map mapView : MKMapView, Controller controller: ServicesMapController) {
        var parsedServices : Array<Service> = Array()
        
        let authManager = AuthenticationManager.getInstance()
        
        let urlComponents = NSURLComponents(string: "http://localhost:8080/secured/service")
        
        let minLatitudeQueryItem = URLQueryItem(name: "minLatitude", value: updateServicesRequest.minLatitude.description)
        let maxLatitudeQueryItem = URLQueryItem(name: "maxLatitude", value: updateServicesRequest.maxLatitude.description)
        let minLongitudeQueryItem = URLQueryItem(name: "minLongitude", value: updateServicesRequest.minLongitude.description)
        let maxLongitudeQueryItem = URLQueryItem(name: "maxLongitude", value: updateServicesRequest.maxLongitude.description)
        let serviceTypeQueryItem = URLQueryItem(name: "serviceType", value: updateServicesRequest.serviceType.description)
        let queryItems :[URLQueryItem] = [minLatitudeQueryItem, maxLatitudeQueryItem, minLongitudeQueryItem, maxLongitudeQueryItem, serviceTypeQueryItem]
        
        urlComponents?.queryItems = queryItems
        let url : URL = (urlComponents?.url)!
        
        
        let request = authManager.oAuthPasswordGrant.request(forURL: url)
        
        authManager.oAuthDataLoader.perform(request: request) { (response) in
            do {
                if (response.error == nil) {
                    let availableServices = try JSONSerialization.jsonObject(with: response.data!, options: JSONSerialization.ReadingOptions.allowFragments) as! [String : AnyObject]
                    
                    if let servicesContent = availableServices["services"] {
                        if servicesContent.count > 0 {
                            for index in 0...servicesContent.count-1 {
                                let serviceAsJson = (servicesContent as! NSArray)[index] as! [String : AnyObject]
                                
                                let service : Service = ServiceJSONParser.parseJsonToService(SerlocServiceFields: serviceAsJson)
                                parsedServices.append(service)
                            }
                        }
                    }
                    
                    
                    DispatchQueue.main.async {
                        var newMarkers : [ServiceMapMark] = [ServiceMapMark]()
                        
                        for service in parsedServices {
                            
                            let serviceLongitude = CLLocationDegrees(service.longitude)
                            let serviceLatitude = CLLocationDegrees(service.latitude)
                            
                            let serviceCoordinate : CLLocationCoordinate2D = CLLocationCoordinate2D(latitude: serviceLatitude, longitude: serviceLongitude)
                            
                            let serviceMark = ServiceMapMark(id: service.id, coordinate: serviceCoordinate, title: service.name, subtitle: service.description, price: service.price)
                            newMarkers.append(serviceMark)
                        }
                        
                        let actualMarkers = mapView.annotations
                        
                        for marker in actualMarkers {
                            mapView.removeAnnotation(marker)
                        }
                        
                        mapView.addAnnotations(newMarkers)
                        
                        controller.servicesArray = parsedServices
                    }
                    
                }
            } catch let error {
                print("Error trying to obtain geolocated user services \(error.asOAuth2Error.description)")
            }
        }
    }
    
    /**
    * Function which performs a request to backend
    * to create a new service definition.
    */
    public static func createNewService(Request request : CreateServiceRequest, Controller controller : AddUserServiceController) {
        let authManager = AuthenticationManager.getInstance()
        let jsonEncoder : JSONEncoder = JSONEncoder()
    
        let url = URL(string: "http://localhost:8080/secured/service")
        var httpRequest = authManager.oAuthPasswordGrant.request(forURL: url!)
        httpRequest.httpMethod = "POST"
        httpRequest.httpBody = try! jsonEncoder.encode(request)
        httpRequest.setValue("application/json", forHTTPHeaderField: "Content-Type")
        
        authManager.oAuthDataLoader.perform(request: httpRequest) { (response) in
            if (response.error == nil) {
                DispatchQueue.main.async {
                    controller.uploadAndRedirectToUserServicesList()
                }
            }
        }
    }
    
    /**
    * Update the user services list for the user services list view.
    */
    public static func updateUserServices(Controller controller : UserServicesListController) {
        var parsedServices = [Service]()
        
        let authManager = AuthenticationManager.getInstance()
        let getUserServicesUrl = URL(string: "http://localhost:8080/secured/user/services")
        
        let request = authManager.oAuthPasswordGrant.request(forURL: getUserServicesUrl!)
        
        authManager.oAuthDataLoader.perform(request: request) { (response) in
            do {
                //The request don't produce error
                if response.error == nil {
                    let availableServices = try JSONSerialization.jsonObject(with: response.data!, options: JSONSerialization.ReadingOptions.allowFragments) as! [String : AnyObject]
                    
                    if let servicesContent = availableServices["services"] {
                        if servicesContent.count > 0 {
                            for index in 0...servicesContent.count-1 {
                                let serviceAsJson = (servicesContent as! NSArray)[index] as! [String : AnyObject]
                                
                                let service : Service = ServiceJSONParser.parseJsonToService(SerlocServiceFields: serviceAsJson)
                                parsedServices.append(service)
                            }
                        }
                    }
                } else  {
                    print("Http error gettin the user services.")
                }
                
                DispatchQueue.main.async {
                    controller.updateServicesList(parsedServices)
                }
                
            } catch let error {
                print("Error managing the server response \(error.asOAuth2Error.description)")
            }
        }
    }
}
