import UIKit
import MapKit

class ServiceDetailController: UIViewController, MKMapViewDelegate {

    
    @IBOutlet weak var serviceLocation: MKMapView!
    @IBOutlet weak var serviceDescription: UITextView!
    @IBOutlet weak var serviceProviderName: UITextView!
    @IBOutlet weak var servicePrice: UITextView!
    @IBOutlet weak var serviceTitle: UITextView!
    
    public var serviceDetail : Service?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.tabBarController?.tabBar.isHidden = true
        
        self.configureMapView();
        self.completeServiceInformation();
    

        // Do any additional setup after loading the view.
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    /**
    * Function which perform segue to call service provider.
    */
    @IBAction func callServiceProvider(_ sender: Any) {
        guard let serviceProviderNumber = URL(string: "tel://" + (serviceDetail?.serviceProvider.phone)!) else { return }
        UIApplication.shared.open(serviceProviderNumber, options: [:], completionHandler: nil)
    }
    
    /**
    * Configure the map view instance.
    */
    private func configureMapView() {
        self.serviceLocation.isZoomEnabled = false
        self.serviceLocation.isScrollEnabled = false
        self.serviceLocation.isPitchEnabled = false
        self.serviceLocation.isRotateEnabled = false
        
        self.serviceLocation.delegate = self
        self.serviceLocation.showsUserLocation = true
        
        let serviceLatitude : Double = Double((serviceDetail?.latitude)!)
        let serviceLongitude : Double = Double((serviceDetail?.longitude)!)
        let serviceCoordinate = CLLocationCoordinate2D(latitude: serviceLatitude, longitude: serviceLongitude)
        let serviceCoordinateRegion = MKCoordinateRegionMakeWithDistance(serviceCoordinate, 500, 500);
        self.serviceLocation.setRegion(serviceCoordinateRegion, animated: true)
        
        addServiceMark()
    }
    
    /**
     * Add service mark.
     */
    private func addServiceMark() {
        let serviceLongitude = CLLocationDegrees((serviceDetail?.longitude)!)
        let serviceLatitude = CLLocationDegrees((serviceDetail?.latitude)!)
        
        let serviceCoordinate : CLLocationCoordinate2D = CLLocationCoordinate2D(latitude: serviceLatitude, longitude: serviceLongitude)
        
        let serviceMark = ServiceMapMark(id: (serviceDetail?.id)!, coordinate: serviceCoordinate, title: (serviceDetail?.name)!, subtitle: (serviceDetail?.description)!, price: (serviceDetail?.price)!)
        
        self.serviceLocation.addAnnotation(serviceMark)
    }
    
    /**
    * Complete the service information view.
    */
    private func completeServiceInformation() {
        
        self.serviceTitle.text = serviceDetail?.name
        self.servicePrice.text = SerlocFormatter.formatPrice(Price: (serviceDetail?.price.description)!)
        self.serviceProviderName.text = serviceDetail?.serviceProvider.name
        self.serviceDescription.text = serviceDetail?.description
    }
}
