import UIKit
import CoreLocation

class ServiceTypeListController: UIViewController {

    var selectedServiceType : Int?
    public var availableServices : Array<Service> = Array<Service>()
    
    /** Service types buttons */
    @IBOutlet weak var reformsButton: UIButton!
    @IBOutlet weak var transportButton: UIButton!
    @IBOutlet weak var homeButton: UIButton!
    @IBOutlet weak var technologiesButton: UIButton!
    @IBOutlet weak var careButton: UIButton!
    @IBOutlet weak var professorsButton: UIButton!
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.title = "Services Types"

        // Do any additional setup after loading the view.
    }

    @IBAction func serviceTypeSelected(_ sender: UIButton) {
        switch sender {
        case reformsButton:
            searchServices(ServiceType: ServicesTypes.reforms)
        case transportButton:
            searchServices(ServiceType: ServicesTypes.transport)
        case homeButton:
            searchServices(ServiceType : ServicesTypes.home)
        case technologiesButton:
            searchServices(ServiceType: ServicesTypes.technologies)
        case careButton:
            searchServices(ServiceType: ServicesTypes.care)
        case professorsButton:
            searchServices(ServiceType: ServicesTypes.professors)
        default:
            return
        }
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    /**
    *   Function which tryies to obtain the current user location
    *   and obtain all the available services for the
    */
    private func searchServices(ServiceType serviceType : ServicesTypes) {
        
        self.selectedServiceType = serviceType.rawValue
        
        let actualUserLocation = GeolocationHelper.getCurrentUserLocation()
        
        let latitude : CLLocationDegrees = actualUserLocation.coordinate.latitude
        let longitude : CLLocationDegrees = actualUserLocation.coordinate.longitude
        
        let latitudeAsFloat = Float(latitude)
        let longitudeAsFloat = Float(longitude)
        
        /** Request used to call the infraestructure component */
        let getServicesRequest = ServicesRequest(Latitude: latitudeAsFloat, Longitude: longitudeAsFloat, ServiceType: serviceType.rawValue)
        SerlocServiceConsumer.getServicesBy(Request: getServicesRequest, ServiceTypeController : self)

    }
    
    /**
    * Function used to redirect flow to the available services list.
    */
    public func goToServicesList(ServicesList services : Array<Service>) {
        self.availableServices = services
        self.performSegue(withIdentifier: "showServicesList", sender: self)
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if (segue.identifier == "showServicesList") {
            (segue.destination as! ServicesListController).services = availableServices
            (segue.destination as! ServicesListController).servicesType = self.selectedServiceType
        }
    }
}
