import UIKit

class ServicesListController: UIViewController, UITableViewDataSource, UITableViewDelegate {

    var servicesType : Int!
    var services : Array<Service>!
    private var servicesArray : [Service] = [Service]()
    
    @IBOutlet weak var servicesTable: UITableView!
    @IBOutlet weak var mapButton: UIButton!
    @IBOutlet weak var noAvailableServicesMessage: UITextView!
    
    var selectedService : Service?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        if (services.count < 1) {
            self.servicesTable.isHidden = true
            //self.mapButton.isHidden = true
            
            self.noAvailableServicesMessage.isHidden = false
        }
        
        servicesArray.reserveCapacity(services.count)
        
        services.forEach { (service) in
            servicesArray.append(service)
        }
        
        servicesTable.dataSource = self
        servicesTable.delegate = self
    }
    
    /**
     * Recovering tab bar if returning from service detail view.
     */
    override func viewWillAppear(_ animated: Bool) {
        self.tabBarController?.tabBar.isHidden = false
    }
    
    /**
    * Modifying the segue preparatio to add arguments.
    */
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if (segue.identifier == "showServicesMap") {
            (segue.destination as! ServicesMapController).services = services
            (segue.destination as! ServicesMapController).previousSelectedServiceType = self.servicesType
        }
        if (segue.identifier == "showServiceDetail") {
            (segue.destination as! ServiceDetailController).serviceDetail = selectedService!
        }
    }
    
    @IBAction func goToServicesMap(_ sender: Any) {
        self.performSegue(withIdentifier: "showServicesMap", sender: self)
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
        
        self.servicesTable.register(ServiceCell.self, forCellReuseIdentifier: "ServiceCell")
    }
    
    /**
    * Function which determines the number of sections of the table.
    */
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    
    /**
    * Function to determine the number of the rows of the table
    */
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return servicesArray.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let serviceCell = tableView.dequeueReusableCell(withIdentifier: "ServiceCell", for: indexPath) as! ServiceCell

        let service : Service = self.servicesArray[indexPath.row]
        
        serviceCell.serviceTitle.text = service.name
        serviceCell.serviceDescription.text = service.description
        serviceCell.servicePrice.text = SerlocFormatter.formatPrice(Price: service.price.description)
        
        self.services.append(service)
        
        return serviceCell
    }
    
    /**
     * Function which process the services selected operation.
     */
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        self.selectedService = self.servicesArray[indexPath.row]
        self.performSegue(withIdentifier: "showServiceDetail", sender: self)
    }
    
    // Set the spacing between sections
    func tableView(_ tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        return 30
    }
    
    func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        let separator : UIView = UIView()
        separator.backgroundColor = UIColor.white.withAlphaComponent(0.5)
        return separator
    }
    
    /**
    * Table view delegate function which returns the cell height. The serloc service cell height must be constant.
    */
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 199
    }

}
