import Foundation

/**
 * User entity definition.
 */
class User : IdentifiedEntity, Codable {
    
    public var name : String
    public let email : String
    public var password : String
    public var description : String
    public var phone : String
    
    /**
    * Base user initialization with the basic information.
    * Used to generate new user instances.
    */
    init (Name name : String,
          Email email : String,
          Password password : String,
          PhoneNumber phone : String) {
        self.name = name
        self.email = email
        self.password = password
        self.phone = phone
        self.description = Constants.USER_DESCRIPTION_EMPTY
        super.init()
    }
    
    /**
    * Constructor for the new based http request
    */
    init (Id id : Int,
          Name name : String,
          Email email : String,
          Description description : String?,
          PhoneNumber phone : String) {
        self.name = name
        self.email = email
        self.phone = phone
        self.password = Constants.EMPTY_PASSWORD
        if let descriptionWithContent = description {
            self.description = descriptionWithContent
        } else {
            self.description = Constants.USER_DESCRIPTION_EMPTY
        }
        super.init(Id : id)
    }
}
