import UIKit

class UserServicesListController: UIViewController, UITableViewDataSource, UITableViewDelegate {

    public var services : [Service]?
    
    @IBOutlet weak var noAvailableServicesMessage: UITextView!
    @IBOutlet weak var servicesTable: UITableView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        let addServiceButton = UIBarButtonItem(barButtonSystemItem: .add, target: self, action: #selector(goToAddServiceView))
        navigationItem.rightBarButtonItems = [addServiceButton]

        
        if ((services?.count)! < 1) {
            self.servicesTable.isHidden = true
            self.noAvailableServicesMessage.isHidden = false
        } else {
            self.servicesTable.isHidden = false
            self.noAvailableServicesMessage.isHidden = true
        }
        
        self.servicesTable.dataSource = self
        self.servicesTable.delegate = self

        self.title = "My services"
        // Do any additional setup after loading the view.
    }
    
    override func viewWillAppear(_ animated: Bool) {
        SerlocServiceConsumer.updateUserServices(Controller: self)
    }
    
    /**
    * Function to perform segueu which redirects to add service view.
    */
    @objc func goToAddServiceView() {
        self.performSegue(withIdentifier: "showAddServiceView", sender: self)
    }
    
    /**
     * Function which determines the number of sections of the table.
     */
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    
    /**
     * Function to determine the number of the rows of the table
     */
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return services!.count
    }
    
    /**
    * Service table view inflater method.
    */
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let serviceCell = tableView.dequeueReusableCell(withIdentifier: "ServiceCell", for: indexPath) as! ServiceCell
        
        let service : Service = self.services![indexPath.item]
        
        serviceCell.serviceTitle.text = service.name
        serviceCell.serviceDescription.text = service.description
        serviceCell.servicePrice.text = SerlocFormatter.formatPrice(Price: service.price.description)
        
        return serviceCell
    }
    
    /**
     * Table view delegate function which returns the cell height. The serloc service cell height must be constant.
     */
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 199
    }
    
    /**
    * Update the list with the new obtained user services.
    */
    public func updateServicesList(_ updatedServices : [Service]) {
        self.services = updatedServices
        self.servicesTable.reloadData()
    }
    

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

}
